/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules;

import com.keyware.sonar.cxx.CxxLanguage;
import com.keyware.sonar.cxx.LogLanguage;
import org.sonar.api.server.profile.BuiltInQualityProfilesDefinition;
import org.sonarsource.api.sonarlint.SonarLintSide;

/**
 * TODO SecurityDesignWayProfile
 *
 * @author GuoXin
 * @date 2024/1/12
 */
@SonarLintSide
public class SecurityDesignWayProfile implements BuiltInQualityProfilesDefinition {
    @Override
    public void define(Context context) {
        var way = context.createBuiltInQualityProfile("C++信息安全性设计准则", CxxLanguage.KEY);
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "ABCVarNameChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "BufferDataChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "CmdDataVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "DLLVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "EncryptionAlgorithmChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "ErrorMessageChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "FileAccessChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "FormatFunctionCheck");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "FVNRPassWordChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "FVNRShaChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "HighEncryptDesChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "HostIdentityVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "IntegerCountVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "LogFileWriteChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "NumericalCopyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "PassWordCountChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "PathVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "PRNGVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "ReallocMainChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "SendMessageChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "FlagLine1Rule");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "SQLVerifyChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "UserInputPasswordChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "ValidatePasswordCheck");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "VerificationPathChecker");
        way.activateRule(SecurityDesignRuleRepository.REPOSITORY_KEY, "VirtualLockUsageChecker");
        way.done();

        var way1 = context.createBuiltInQualityProfile("LogLanguage信息安全性设计准则", LogLanguage.KEY);
        way1.activateRule(SecurityDesignRuleRepository.REPOSITORY_log_key, "LogChecker");
        way1.done();
    }
}
