/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;



import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;

import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import java.util.HashMap;
import java.util.Map;

/**
 * TODO 应使用单向加密算法对口令进行加密并存储
 *
 * @author WuHaoYang
 * @date 2024/1/18
 */
@Rule(key = "EncryptionAlgorithmChecker", name = "应使用单向加密算法对口令进行加密并存储", description = "应使用单向加密算法对口令进行加密并存储", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class EncryptionAlgorithmChecker extends SquidCheck<Grammar> {

    private Map<String, AstNode> cache = new HashMap<>();

    @Override
    public void init() {
        subscribeTo(
                CxxGrammarImpl.declarationStatement
        );
    }

    @Override
    public void visitNode(AstNode astNode) {
        var varName = astNode.getFirstDescendant(CxxGrammarImpl.declaratorId).getTokenOriginalValue();
        if(varName.contains("password") && !cache.containsKey(varName)){
            cache.put(varName, astNode);
            var next = astNode.getNextAstNode();
            while (next != null){
                var callNode = next.getFirstDescendant(CxxGrammarImpl.postfixExpression);
                if(callNode != null){
                    var callList = callNode.getDescendants(CxxGrammarImpl.className);
                    if(!callList.isEmpty()) {
                        var funName = callList.get(callList.size() - 1).getTokenOriginalValue();
                        var paramList = callNode.getDescendants(CxxGrammarImpl.expressionList);
                        if("hashpw".equalsIgnoreCase(funName) && paramList.stream().anyMatch(item-> {
                            var name = item.getTokenOriginalValue();
                            return name != null && ("password".equals(name) || "salt".equals(name));
                        })){
                            cache.remove(varName);
                        }
                    }
                }
                next = next.getNextSibling();
            }
            cache.values().forEach(item->{
                getContext().createLineViolation(this, "特定字段未使用单向加密算法对口令进行加密并存储", item);
            });
        }
    }
}
