/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;


import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import java.util.ArrayList;
import java.util.List;

import static com.sonar.cxx.sslr.api.GenericTokenType.IDENTIFIER;

@Rule(key = "VerificationPathChecker", name = "在构建路径名前对数据进行校验", description = "对输入数据进行校验", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class VerificationPathChecker extends SquidCheck<Grammar> {

    @Override
    public void init() {
        // 订阅要检查AST节点类型，用于在visitNode方法中检查该类型节点
        this.subscribeTo(
                CxxGrammarImpl.functionBody
        );
    }
    public void visitNode(AstNode astNode) {
        List<AstNode> descendants = astNode.getDescendants(CxxGrammarImpl.statement);
        for (AstNode ast:descendants) {
            List<AstNode> descendants1 = ast.getDescendants(CxxGrammarImpl.shiftExpression);
            for (AstNode desc :descendants1) {
                if("cin".equals(desc.getTokenValue())){
                    List<AstNode> children = desc.getChildren();
                    for (AstNode chil:children) {
                        if("IDENTIFIER".equals(chil.getName())){
                            if(chil.getTokenValue().toLowerCase().contains("path")){
                                boolean boo = true;
                                List<AstNode> exprs = astNode.getDescendants(CxxGrammarImpl.postfixExpression);
                                for (AstNode expr:exprs) {
                                    if(expr.getTokenValue().contains("check") || expr.getTokenValue().contains("verify") || expr.getTokenValue().contains("valid")){
                                        List<AstNode> astNodeList = expr.getDescendants(CxxGrammarImpl.expressionList);
                                        for (AstNode asrList:astNodeList) {
                                            if(chil.getTokenValue().equals(asrList.getTokenValue())){
                                                boo = false;
                                            }
                                        }
                                    }
                                }
                                if(boo){
                                    getContext().createLineViolation(this, "在构建路径名前对数据进行校验", chil);
                                }
                            }
                        }
                    }
                }

            }


        }

    }
}
