/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;

import com.keyware.sonar.cxx.CxxFileTesterHelper;
import org.junit.jupiter.api.Test;
import org.sonar.cxx.CxxAstScanner;
import org.sonar.cxx.squidbridge.api.SourceFile;
import org.sonar.cxx.squidbridge.checks.CheckMessagesVerifier;

import java.io.IOException;

/**
 * 测试规则：用户输入口令时对口令域进行掩饰，通常，用户输入的每一个字符都应该以“*”形式回显
 *
 * @author GuoXin
 * @date 2024/1/20
 */
public class UserInputPasswordCheckerTest {
    @Test
    public void checkTest() throws IOException {
        var checker = new UserInputPasswordChecker();
        var tester = CxxFileTesterHelper.create("UserInputPasswordChecker.cc");
        SourceFile file = CxxAstScanner.scanSingleInputFile(tester.asInputFile(), checker);
        CheckMessagesVerifier.verify(file.getCheckMessages())
                .next().atLine(18).withMessage("用户输入口令时对口令域进行掩饰，通常，用户输入的每一个字符都应该以“*”形式回显")
                .next().atLine(22).withMessage("用户输入口令时对口令域进行掩饰，通常，用户输入的每一个字符都应该以“*”形式回显")
                .noMore();
    }
}
