/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.html.checks.AbstractPageCheck;
import org.sonar.plugins.html.node.TagNode;
import org.sonar.plugins.java.api.JavaCheck;

/**
 * <p>规则：用户输入口令时应对口令域进行掩饰。用户输入的每一个字符都应该以星号形式回显。</p>
 * 验证逻辑：当input标签的name或者id包含password时，如果type不是password时，则报告问题
 *
 * @author GuoXin
 * @date 2024/1/20
 */
@Rule(key = "PasswordInputTagChecker")
public class PasswordInputTagChecker extends AbstractPageCheck implements JavaCheck {
    @Override
    public void startElement(TagNode node) {
        if ("input".equalsIgnoreCase(node.getNodeName())) {
            var id = node.getAttribute("id");
            var name = node.getAttribute("name");
            var type = node.getAttribute("type");
            // 当name或者id包含password时，如果type不是password时，则报告问题
            if ((id != null && id.contains("password")) || (name != null && name.contains("password"))) {
                if (!"password".equalsIgnoreCase(type)) {
                    createViolation(node, "密码输入框的type属性不为password");
                }
            }
        }
    }
}
